'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.done = exports.fulfilled = exports.rejected = exports.pending = exports.getDoneState = exports.getFulfilledState = exports.getRejectedState = exports.getPendingState = exports.getThunkState = exports.initialState = exports.DONE = exports.FULFILLED = exports.REJECTED = exports.PENDING = undefined;

var _initialState;

var _find = require('lodash/find');

var _find2 = _interopRequireDefault(_find);

var _pick = require('lodash/pick');

var _pick2 = _interopRequireDefault(_pick);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var PENDING = exports.PENDING = 'pending';
var REJECTED = exports.REJECTED = 'rejected';
var FULFILLED = exports.FULFILLED = 'fulfilled';
var DONE = exports.DONE = 'done';

var initialState = exports.initialState = (_initialState = {}, _defineProperty(_initialState, PENDING, {}), _defineProperty(_initialState, REJECTED, {}), _defineProperty(_initialState, FULFILLED, {}), _defineProperty(_initialState, DONE, {}), _initialState);

var getIn = function getIn(state, name) {
  if (typeof name === 'undefined') {
    return !!(0, _find2.default)(state, function (value) {
      return !!value;
    });
  } else if (Array.isArray(name)) {
    return !!(0, _find2.default)((0, _pick2.default)(state, name), function (value) {
      return !!value;
    });
  } else if (Object.prototype.hasOwnProperty.call(state, name)) {
    return !!state[name];
  }
  return false;
};

var getThunkState = exports.getThunkState = function getThunkState() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  if (!Object.prototype.hasOwnProperty.call(state, 'thunk')) {
    throw new Error('[redux-saga-thunk] There is no thunk state on reducer');
  }
  return state.thunk || {};
};

var getPendingState = exports.getPendingState = function getPendingState(state) {
  return getThunkState(state)[PENDING] || initialState[PENDING];
};

var getRejectedState = exports.getRejectedState = function getRejectedState(state) {
  return getThunkState(state)[REJECTED] || initialState[REJECTED];
};

var getFulfilledState = exports.getFulfilledState = function getFulfilledState(state) {
  return getThunkState(state)[FULFILLED] || initialState[FULFILLED];
};

var getDoneState = exports.getDoneState = function getDoneState(state) {
  return getThunkState(state)[DONE] || initialState[DONE];
};

/**
 * Tells if an action is pending
 * @example
 * const mapStateToProps = state => ({
 *   fooIsPending: pending(state, 'FOO'),
 *   fooOrBarIsPending: pending(state, ['FOO', 'BAR']),
 *   anythingIsPending: pending(state)
 * })
 */
var pending = exports.pending = function pending(state, name) {
  return getIn(getPendingState(state), name);
};

/**
 * Tells if an action was rejected
 * @example
 * const mapStateToProps = state => ({
 *   fooWasRejected: rejected(state, 'FOO'),
 *   fooOrBarWasRejected: rejected(state, ['FOO', 'BAR']),
 *   anythingWasRejected: rejected(state)
 * })
 */
var rejected = exports.rejected = function rejected(state, name) {
  return getIn(getRejectedState(state), name);
};

/**
* Tells if an action is fulfilled
* @example
* const mapStateToProps = state => ({
*   fooIsFulfilled: fulfilled(state, 'FOO'),
*   fooOrBarIsFulfilled: fulfilled(state, ['FOO', 'BAR']),
*   anythingIsFulfilled: fulfilled(state)
* })
*/
var fulfilled = exports.fulfilled = function fulfilled(state, name) {
  return getIn(getFulfilledState(state), name);
};

/**
* Tells if an action is done
* @example
* const mapStateToProps = state => ({
*   fooIsDone: done(state, 'FOO'),
*   fooOrBarIsDone: done(state, ['FOO', 'BAR']),
*   anythingIsDone: done(state)
* })
*/
var done = exports.done = function done(state, name) {
  return getIn(getDoneState(state), name);
};